#include "serial.h"
#include "crc.h"
#include "debug.h"
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <termios.h>
#include <sys/poll.h>

int serial_fd;

/* Open and initialize serial port.
   Returns 0 on success, or -1 if error */
int serial_init(char *port)
{
	int fd;
	struct termios t;

	if((serial_fd=open(port,O_RDWR,O_NONBLOCK))<0)
		return -1;
	if(tcgetattr(serial_fd,&t)<0)
		return -1;
	t.c_cflag |= CLOCAL;
	if(tcsetattr(serial_fd,TCSANOW,&t)<0)
		return -1;
	
	bzero(&t,sizeof(t));
	t.c_iflag=IGNBRK|IGNPAR;
	t.c_cflag=CS8|CREAD|CLOCAL;
	t.c_cc[VMIN]=1;
	if(cfsetispeed(&t,B9600)==-1)
		return -1;
	if(tcsetattr(serial_fd,TCSANOW,&t)==-1)
		return -1;

	return 0;
}

void serial_flush(void) {
  char junk[1024];
  fcntl(serial_fd, F_SETFL, fcntl(serial_fd, F_GETFL, 0) | O_NONBLOCK);
  while(read(serial_fd,junk,1024)>0)
    continue;
  fcntl(serial_fd, F_SETFL, fcntl(serial_fd, F_GETFL, 0) & ~O_NONBLOCK);
}

/* Close a serial port */
void serial_close(void)
{
	close(serial_fd);
}

/* returns -1 on timeout/error, otherwise 0 */
int serial_getnextbyte(char *ch, int timeout_msec)
{
	struct pollfd pfd;

	pfd.fd = serial_fd;
	pfd.events = POLLIN|POLLERR;

	/* Wait 2 seconds for the byte */
	if(poll(&pfd, 1, timeout_msec)!=1)
		return -1;
	if(pfd.revents & POLLERR)
		return -1;
	
	if(read(serial_fd, ch, 1)!=1)
		return -1;

	return 0;
}

/* Read a packet from the serial port and return it.
   Return 0 on success, or -1 on error/timeout */
int serial_getpacket(_sPacket *p, int timeout_msec)
{
	_sProcessorInfo *pi = ProcessorInit();
	char ch;

	p->cpPayload=NULL;

	if(pi==NULL)
		return -1;

 more:
	if(serial_getnextbyte(&ch,timeout_msec)<0) {
		/* timeout */
		return -1;
	}
	if(ProcessByte(pi, ch)!=1)
		goto more;

	memcpy(p,&pi->packet,sizeof(_sPacket));
	free(pi);
	
	packet_received(p);

	return 0;
}

void serial_write(char c)
{
	write(serial_fd,&c,1);
}

void serial_escape_write(char c)
{
	if(c==0x10) serial_write(c);
	serial_write(c);
}
	
/* Send a packet. */
void serial_sendpacket(_sPacket *p)
{
	int i;

	serial_write(0x10);
	serial_write(p->cPacketType);
	serial_escape_write(p->iLen >> 8);
	serial_escape_write(p->iLen & 0xff);
	serial_escape_write(p->cSeq);
	serial_escape_write(p->cID);
	for(i=0;i<p->iLen-2;i++)
		serial_escape_write(p->cpPayload[i]);
	serial_escape_write(p->iCRC >> 8);
	serial_escape_write(p->iCRC & 0xff);
	serial_write(0x10);
	serial_write(0x03);

	packet_sent(p);
}
